#####################################
###  a function for generating simulation 
###   dataof negative binomial distribution
#####################################
NBsample_multi <- function(DEG.foldchange = c(4, 4), group = c(3, 3), Ngene = 1000, PDEG = 0.20, DEG.assign = c(0.9, 0.1)) 
{
    arab <- NULL
    rm(arab)
    data(arab)
    data.cl <- c(rep(1, 3), rep(2, 3))
    RPM <- sweep(arab, 2, 1e+06/colSums(arab), "*")
    RPM_A <- RPM[, data.cl == 1]
    RPM_B <- RPM[, data.cl == 2]
    RPM_A <- RPM_A[apply(RPM_A, 1, var) > 0, ]
    RPM_B <- RPM_B[apply(RPM_B, 1, var) > 0, ]
    MEAN <- c(apply(RPM_A, 1, mean), apply(RPM_B, 1, mean))
    VARIANCE <- c(apply(RPM_A, 1, var), apply(RPM_B, 1, var))
    DISPERSION <- (VARIANCE - MEAN)/(MEAN * MEAN)
    mean_disp_tmp <- cbind(MEAN, DISPERSION)
    mean_disp_tmp <- mean_disp_tmp[mean_disp_tmp[, 2] > 0, ]
    resampling_vector <- sample(1:nrow(mean_disp_tmp), Ngene, 
        replace = TRUE)
    mean_disp <- mean_disp_tmp[resampling_vector, ]
    mu <- mean_disp[, 1]

    out_tmp <- NULL
    for(i in 1:sum(group)) {
        out_tmp <- cbind(out_tmp, rnbinom(n = length(mu), mu = mu, size = 1/mean_disp[,2]))
    }
    tmp1 <- rep(LETTERS[1:length(group)], times = group)
    tmp2 <- as.character(sequence(group))
    colnames(out_tmp) <- paste(tmp1, tmp2, sep="")

    DEG_posi <- as.character(rep(0, Ngene))
    DEG_posi[1:(Ngene*PDEG)] <- rep(LETTERS[1:length(DEG.assign)], times = Ngene*PDEG*DEG.assign)

    out <- out_tmp
    cl <- LETTERS[1:length(group)]
    for(i in 1:length(cl)) {
        p_row <- DEG_posi == cl[i]
        p_col <- tmp1 == cl[i]
        out[p_row, p_col] <- out[p_row, p_col] * DEG.foldchange[i]
    }
    rownames(out) <- paste("gene", 1:nrow(out), sep="_")

    retval <- list(
              count = out,
              truthDEG = DEG_posi,
           )
    return(retval)
}


#####################################
###  a function for generating simulation 
###   dataof negative binomial distribution
#####################################
NBsample2 <- function(DEG_foldchange = 4, repA = 3, repB = 3, Ngene = 3000, PDEG = 0.15, PA = 0.2) 
{
    arab <- NULL
    rm(arab)
    data(arab)
    data.cl <- c(rep(1, 3), rep(2, 3))
    RPM <- sweep(arab, 2, 1e+06/colSums(arab), "*")
    RPM_A <- RPM[, data.cl == 1]
    RPM_B <- RPM[, data.cl == 2]
    RPM_A <- RPM_A[apply(RPM_A, 1, var) > 0, ]
    RPM_B <- RPM_B[apply(RPM_B, 1, var) > 0, ]
    MEAN <- c(apply(RPM_A, 1, mean), apply(RPM_B, 1, mean))
    VARIANCE <- c(apply(RPM_A, 1, var), apply(RPM_B, 1, var))
    DISPERSION <- (VARIANCE - MEAN)/(MEAN * MEAN)
    mean_disp_tmp <- cbind(MEAN, DISPERSION)
    mean_disp_tmp <- mean_disp_tmp[mean_disp_tmp[, 2] > 0, ]
    resampling_vector <- sample(1:nrow(mean_disp_tmp), Ngene, 
        replace = TRUE)
    mean_disp <- mean_disp_tmp[resampling_vector, ]
    mu <- mean_disp[, 1]
    DEG_degree_A <- rep(1, Ngene)
    DEG_degree_A[1:(Ngene * PDEG * PA)] <- DEG_foldchange
    mu_A <- mu * DEG_degree_A
    DEG_degree_B <- rep(1, Ngene)
    DEG_degree_B[(Ngene * PDEG * PA + 1):(Ngene * PDEG)] <- DEG_foldchange
    mu_B <- mu * DEG_degree_B
    DEG_posi_org <- (DEG_degree_A * DEG_degree_B) > 1
    nonDEG_posi_org <- (DEG_degree_A * DEG_degree_B) == 1
    outA <- NULL
    colnamev <- NULL
    for (i in 1:repA) {
        outA <- cbind(outA, rnbinom(n = length(mu_A), mu = mu_A, size = 1/mean_disp[, 2]))
        colnamev <- cbind(colnamev, paste("A", as.character(i), sep = ""))
    }
    outB <- NULL
    for (i in 1:repB) {
        outB <- cbind(outB, rnbinom(n = length(mu_B), mu = mu_B, size = 1/mean_disp[, 2]))
        colnamev <- cbind(colnamev, paste("B", as.character(i), sep = ""))
    }
    out <- cbind(outA, outB)
    colnames(out) <- colnamev
    obj <- rowSums(out) > 0
    RAW <- out[obj, ]
    DEG_posi <- DEG_posi_org[obj]
    nonDEG_posi <- nonDEG_posi_org[obj]

    retval <- list(RAW, DEG_posi, nonDEG_posi)
    names(retval) <- c("data", "DEG_posi", "nonDEG_posi")
    return(retval)
}

#####################################
###  DESeq DES normalization method
#####################################
do_DDD <- function(data=NULL, data.cl=NULL, trim=NULL){
    RAW <- data
    ###  Step 1: first normalization
    cds <- newCountDataSet(data, data.cl)
    cds <- estimateSizeFactors(cds)
    norm_f_DESeq <- sizeFactors(cds)

    ###  Step 2: DEG identification by FDR threshold
    #if(length(data.cl) == 2){
        cds <- estimateDispersions(cds, method="blind", sharingMode="fit-only", fitType="local")
    out <- nbinomTest(cds, 1, 2)
    #out2 <- out$padj
    #DEG_posi <- (out2 < FDR)
    #nonDEG_posi <- (out2 >= FDR)
    #PDEG <- sum(DEG_posi)/length(out2)
    #PA <- sum(out$log2FoldChange[DEG_posi] < 0)/sum(DEG_posi)
    out2 <- rank(out$pval)
    DEG_posi <- (out2 < length(out2)*trim_value)
    nonDEG_posi <- (out2 >= length(out2)*trim_value)
    PDEG <- sum(DEG_posi)/length(out2)
    PA <- sum(out$log2FoldChange[DEG_posi] < 0)/sum(DEG_posi)

    ###  Step 3: second normalization without DEG
    data <- RAW[nonDEG_posi,]
    cds <- newCountDataSet(data, data.cl)
    cds <- estimateSizeFactors(cds)
    norm_f_DES <- sizeFactors(cds)

    retval <- list(
              norm_f_DES = norm_f_DES,
              PDEG = PDEG,
              PA = PA, 
              nonDEG_posi = nonDEG_posi,
              DEG_posi = DEG_posi, 
              norm_f_DESeq = norm_f_DESeq
           )
    return(retval)
}

#####################################
###  edgeR DES normalization method
#####################################
do_edgeR_DES <- function(data=NULL, data.cl=NULL, FDR=0.1){
    RAW <- data
    ###  Step 1: first normalization
    d <- DGEList(counts=data, group=data.cl)
    d <- calcNormFactors(d)
    norm_f_TMM <- d$samples$norm.factors
    names(norm_f_TMM) <- colnames(data)

    ###  Step 2: DEG identification by FDR threshold
    d <- estimateCommonDisp(d)
    d <- estimateTagwiseDisp(d, prop.used=0.5, grid.length=500)
    out <- exactTest(d)
    out2 <- decideTestsDGE(out, p=FDR, adjust="BH")
    PDEG <- sum(out2 != 0)/length(out2)
    PA <- sum(out2 == 1)/sum(out2 != 0)
    DEG_posi <- (out2 != 0)
    nonDEG_posi <- (out2 == 0)

    ###  Step 3: second normalization without DEG
    data <- RAW[nonDEG_posi,]
    d <- DGEList(counts=data, group=data.cl)
    d <- calcNormFactors(d)
    norm_f_DESorg <- d$samples$norm.factors * colSums(data) / colSums(RAW)
    norm_f_DES <- norm_f_DESorg / mean(c(mean(norm_f_DESorg[data.cl==1]),mean(norm_f_DESorg[data.cl==2])))

    retval <- list(
              norm_f_DES = norm_f_DES,
              PDEG = PDEG,
              PA = PA, 
              nonDEG_posi = nonDEG_posi,
              DEG_posi = DEG_posi, 
              norm_f_TMM = norm_f_TMM,
              norm_f_DESorg = norm_f_DESorg
           )
    return(retval)
}

#####################################
###  DESeq-WAD-DESeq normalization method
#####################################
do_DWD <- function(data=NULL, data.cl=NULL, trim=NULL){
    RAW <- data

    ###  Step 1: first normalization
    cds <- newCountDataSet(data, data.cl)
    cds <- estimateSizeFactors(cds)
    norm_f <- sizeFactors(cds)
    data.tmp <- sweep(RAW, 2, 1/norm_f, "*")
    data.tmp[data.tmp < 1] <- 1
    data_DESeq.log <- log2(data.tmp)

    ###  Step 2: DEG identification
    data <- data_DESeq.log
    out <- WAD(data=data, data.cl=data.cl)
    out2 <- rank(-abs(out))
    DEG_posi <- (out2 < length(out2)*trim_value)
    nonDEG_posi <- (out2 >= length(out2)*trim_value)
    PDEG <- sum(DEG_posi)/length(out2)
    PA <- sum(out[DEG_posi] < 0)/sum(DEG_posi)

    ###  Step 3: second normalization without DEG
    data <- RAW[nonDEG_posi,]
    cds <- newCountDataSet(data, data.cl)
    cds <- estimateSizeFactors(cds)
    norm_f_DES <- sizeFactors(cds)

    retval <- list(
              norm_f_DES = norm_f_DES,
              PDEG = PDEG,
              PA = PA, 
              nonDEG_posi = nonDEG_posi,
              DEG_posi = DEG_posi
           )
    return(retval)
}

#####################################
###  DESeq-WAD-DESeq normalization method (iteration number: 1)
#####################################
do_DWD1 <- function(data=NULL, data.cl=NULL, trim=NULL){
    RAW <- data

    ###  Step 1: first normalization
    cds <- newCountDataSet(data, data.cl)
    cds <- estimateSizeFactors(cds)
    norm_f <- sizeFactors(cds)
    data.tmp <- sweep(RAW, 2, 1/norm_f, "*")
    data.tmp[data.tmp < 1] <- 1
    data_DESeq.log <- log2(data.tmp)

    ###  Step 2: DEG identification
    data <- data_DESeq.log
    out <- WAD(data=data, data.cl=data.cl)
    out2 <- rank(-abs(out))
    DEG_posi <- (out2 < length(out2)*trim_value)
    nonDEG_posi <- (out2 >= length(out2)*trim_value)
    PDEG <- sum(DEG_posi)/length(out2)
    PA <- sum(out[DEG_posi] < 0)/sum(DEG_posi)

    ###  Step 1: first normalization (iteration1)
    data <- RAW[nonDEG_posi,]
    cds <- newCountDataSet(data, data.cl)
    cds <- estimateSizeFactors(cds)
    norm_f <- sizeFactors(cds)
    data.tmp <- sweep(RAW, 2, 1/norm_f, "*")
    data.tmp[data.tmp < 1] <- 1
    data_DESeq.log <- log2(data.tmp)

    ###  Step 2: DEG identification (iteration1)
    data <- data_DESeq.log
    out <- WAD(data=data, data.cl=data.cl)
    out2 <- rank(-abs(out))
    DEG_posi <- (out2 < length(out2)*trim_value)
    nonDEG_posi <- (out2 >= length(out2)*trim_value)
    PDEG <- sum(DEG_posi)/length(out2)
    PA <- sum(out[DEG_posi] < 0)/sum(DEG_posi)


    ###  Step 3: second normalization without DEG
    data <- RAW[nonDEG_posi,]
    cds <- newCountDataSet(data, data.cl)
    cds <- estimateSizeFactors(cds)
    norm_f_DES <- sizeFactors(cds)

    retval <- list(
              norm_f_DES = norm_f_DES,
              PDEG = PDEG,
              PA = PA, 
              nonDEG_posi = nonDEG_posi,
              DEG_posi = DEG_posi 
           )
    return(retval)
}

#####################################
###  DESeq-X-X normalization method
#####################################
do_DXX <- function(data=NULL, data.cl=NULL){
    cds <- newCountDataSet(data, data.cl)
    cds <- estimateSizeFactors(cds)
    norm_f_DES <- sizeFactors(cds)

    retval <- list(
              norm_f_DES = norm_f_DES
           )
    return(retval)
}

#####################################
###  TMM-WAD-TMM normalization method
#####################################
do_TWT <- function(data=NULL, data.cl=NULL, trim=NULL){
    RAW <- data
    ###  Step 1: first normalization
    d <- DGEList(counts=data, group=data.cl)
    d <- calcNormFactors(d)
    norm_f_TMM <- d$samples$norm.factors
    names(norm_f_TMM) <- colnames(data)
    effective_libsizes <- colSums(data) * norm_f_TMM
    data.tmp <- sweep(data, 2, median(colSums(data))/effective_libsizes, "*")
    data_TMM <- data.tmp
    data.tmp[data.tmp < 1] <- 1
    data_TMM.log <- log2(data.tmp)

    ###  Step 2: DEG identification by FDR threshold
    data <- data_TMM.log
    out <- WAD(data=data, data.cl=data.cl)
    out2 <- rank(-abs(out))
    DEG_posi <- (out2 < length(out2)*trim_value)
    nonDEG_posi <- (out2 >= length(out2)*trim_value)
    PDEG <- sum(DEG_posi)/length(out2)
    PA <- sum(out[DEG_posi] < 0)/sum(DEG_posi)

    ###  Step 3: second normalization without DEG
    data <- RAW[nonDEG_posi,]
    d <- DGEList(counts=data, group=data.cl)
    d <- calcNormFactors(d)
    norm_f_DESorg <- d$samples$norm.factors * colSums(data) / colSums(RAW)
    norm_f_DES <- norm_f_DESorg / mean(c(mean(norm_f_DESorg[data.cl==1]),mean(norm_f_DESorg[data.cl==2])))

    retval <- list(
              norm_f_DES = norm_f_DES,
              PDEG = PDEG,
              PA = PA, 
              nonDEG_posi = nonDEG_posi,
              DEG_posi = DEG_posi, 
              norm_f_TMM = norm_f_TMM,
              norm_f_DESorg = norm_f_DESorg
           )
    return(retval)
}

#####################################
###  TMM-X-X normalization method
#####################################
do_TXX <- function(data=NULL, data.cl=NULL){
    d <- DGEList(counts=data, group=data.cl)
    d <- calcNormFactors(d)
    norm_f_DES <- (colSums(data) * d$samples$norm.factors) / median(colSums(data))
    names(norm_f_DES) <- colnames(data)

    retval <- list(
              norm_f_DES = norm_f_DES
           )
    return(retval)
}

#####################################
###  TbT normalization method     ###
#####################################
do_TbT <- function(data, data.cl, sample_num = 10000){
    RAW <- data
    ###  Step 1: first normalization  ###
    d <- DGEList(counts=data, group=data.cl)
    d <- calcNormFactors(d)
    norm_f_TMM <- d$samples$norm.factors
    names(norm_f_TMM) <- colnames(data)

    ###  Step 2: DEG identification  ###
    groups <- list(NDE=rep(1, length(data.cl)), DE=data.cl)
    RPM <- sweep(data, 2, 1000000/colSums(data), "*")
    data <- round(RPM)
    once_normalized <- new("countData", data=as.matrix(data), replicates=data.cl, libsizes=colSums(data)*norm_f_TMM, groups=groups)
    once_normalized.NB <- getPriors.NB(once_normalized, samplesize=sample_num, estimation="QL", cl=NULL)
    out <- getLikelihoods.NB(once_normalized.NB, pET="BIC", cl=NULL)
    PDEG <- out@estProps[2] #proportion of differentially expressed genes
    rank_bayseq <- rank(-out@posteriors[,2])
    NDEG <- (nrow(data) * PDEG)# number of differentially expressed genes

    ###  Step 3: second normalization  ###
    obj_DEGy <- (rank_bayseq < NDEG)
    obj_DEGn <- (rank_bayseq >= NDEG)
    data <- RAW[obj_DEGn,]
    d <- DGEList(counts=data, group=data.cl)
    d <- calcNormFactors(d)
    norm_f_TbTorg <- d$samples$norm.factors*colSums(data)/colSums(RAW)
    norm_f_TbT <- norm_f_TbTorg/mean(c(mean(norm_f_TbTorg[data.cl==1]),mean(norm_f_TbTorg[data.cl==2])))

    ###  calculation of PA value (degree of biased expression)  ###
    RPM_TMM <- sweep(RPM, 2, 1/norm_f_TMM, "*")
    data <- RPM_TMM
    logratio <- log2(apply(data[,data.cl==2], 1, mean)) - log2(apply(data[,data.cl==1], 1, mean))
    PA <- sum(logratio[rank_bayseq < NDEG] < 0)/NDEG

    retval <- list(norm_f_TbT, PDEG, PA, obj_DEGn, obj_DEGy, norm_f_TMM, norm_f_TbTorg)
    names(retval) <- c("norm_f_TbT", "PDEG", "PA", "nonDEG_posi", "DEG_posi", "norm_f_TMM", "norm_f_TbTorg")
    return(retval)
}


#############################
###  Student's t-test     ###
#############################
Students_ttest <- function(x, cl){
    x.class1 <- x[(cl == 1)]                                     #x1̂̂x.class1Ɋi[
    x.class2 <- x[(cl == 2)]                                     #x2̂̂x.class2Ɋi[
    if((sd(x.class1)+sd(x.class2)) == 0){                        #̌Q̕W΍0̏ꍇ͌vZłȂ̂...
        stat <- 0                                                #vʂ0
        pval <- 1                                                #pl1
        return(c(stat, pval))                                    #ƂČʂʂƂĕԂ
    }
    else{                                                        #A, Bǂ炩̌Q̕W΍0(LȊO)̏ꍇ
        hoge <- t.test(x.class1, x.class2, var.equal=T)          #tsāA
        return(c(hoge$statistic, hoge$p.value))                  #vʂplʂƂĕԂ
    }
}

#############################
###  Welth t-test         ###
#############################
Welch_ttest <- function(x, cl){
    x.class1 <- x[(cl == 1)]                                     #x1̂̂x.class1Ɋi[
    x.class2 <- x[(cl == 2)]                                     #x2̂̂x.class2Ɋi[
    if((sd(x.class1)+sd(x.class2)) == 0){                        #̌Q̕W΍0̏ꍇ͌vZłȂ̂...
        stat <- 0                                                #vʂ0
        pval <- 1                                                #pl1
        return(c(stat, pval))                                    #ƂČʂʂƂĕԂ
    }
    else{                                                        #A, Bǂ炩̌Q̕W΍0(LȊO)̏ꍇ
        hoge <- t.test(x.class1, x.class2, var.equal=F)          #tsāA
        return(c(hoge$statistic, hoge$p.value))                  #vʂplʂƂĕԂ
    }
}

#####################################
###  Zpϔłfold change (FC) ###
#####################################
FC <- function(x, cl){
    x <- 2^x                    # logged data -> unlogged dataɕϊiꂪ2łƒf肵Ăj
    x.class1 <- x[(cl == 1)]    #x1̂̂x.class1Ɋi[
    x.class2 <- x[(cl == 2)]    #x2̂̂x.class2Ɋi[
    hoge <- log2(mean(x.class2)/mean(x.class1))
    return(hoge)
}

#########################################################
###  􉽕ϔłfold change (Average Difference; AD) ###
#########################################################
AD <- function(data=NULL, data.cl=NULL){ 
    x <- data
    cl <- data.cl
    mean1 <- rowMeans(as.matrix(x[, cl==1]))
    mean2 <- rowMeans(as.matrix(x[, cl==2]))
    statistic <- (mean2 - mean1)
    return(statistic)
}
#AD <- function(x, cl){
#    x.class1 <- x[(cl == 1)]    #x0̂̂x.class0Ɋi[
#    x.class2 <- x[(cl == 2)]    #x1̂̂x.class1Ɋi[
#    hoge <- mean(x.class2) - mean(x.class1)
#    return(hoge)
#}


###########################################
###  Weighted Average Difference (WAD)  ###
###########################################
WAD <- function(data=NULL, data.cl=NULL){ 
    x <- data
    cl <- data.cl
    mean1 <- rowMeans(as.matrix(x[, cl==1]))
    mean2 <- rowMeans(as.matrix(x[, cl==2]))
    x_ave <- (mean1 + mean2)/2
    weight <- (x_ave - min(x_ave))/(max(x_ave) - min(x_ave))
    statistic <- (mean2 - mean1)*weight
    return(statistic)
}
#WAD <- function(x, cl, dynamic_r, min_v){ 
#    x.class1 <- x[(cl == 0)]
#    x.class2 <- x[(cl == 1)]
#    x_ave <- (mean(x.class1) + mean(x.class2))/2
#    weight <- (x_ave - min_v)/dynamic_r
#    statistic <- (mean(x.class2) - mean(x.class1))*weight
#    return(statistic)
#}


#############################
###  Mann-Whitney U-test  ###
#############################
MW_Utest <- function(x, cl){
    x.class1 <- x[(cl == 1)]    #x1̂̂x.class1Ɋi[
    x.class2 <- x[(cl == 2)]    #x2̂̂x.class2Ɋi[
    hoge <- wilcox.test(x.class1, x.class2)
    return(c(hoge$statistic, hoge$p.value))
}

#############################
###  One-way ANOVA        ###
#############################
Oneway_anova <- function(x, cl){
    hoge <- oneway.test(x ~ cl, var=T)
    return(c(hoge$statistic, hoge$p.value))
}

#############################
###  Kruskal-Wallis test  ###
#############################
Kruskal_wallis <- function(x, cl){
    hoge <- kruskal.test(x ~ cl)
    return(c(hoge$statistic, hoge$p.value))
}

#############################
###  Schug's H statistic  ###
#############################
shannon.entropy <- function(x){ 
    x <- x[(!is.na(x))]    #NAȊO̗vfxNgxɊi[
    x_length <- length(x)  #xNgx̗vfx_lengthɊi[
    x <- x[(x != 0)]       #vf0łȂ̂xɊi[
    if(sum(x) <= 0) {      #x̘a0ȉ̏ꍇ
        return(log(x_length, base=2)) 
    } else {               #ȊȌꍇ
        p <- x/sum(x)      #xNgp̘a1ɂȂ悤ɃxNgx̘ałꂼ̗vf
        score <- -sum(p * log(p, base=2)) 
        return(score)
    }
} 

#############################
###  Schug's Q statistic  ###
#############################
shannon.entropy.q <- function(x){ 
    x_org <- x
    x <- x[(!is.na(x))]    #NAȊO̗vfxNgxɊi[
    x_length <- length(x)  #xNgx̗vfx_lengthɊi[
    x <- x[(x != 0)]       #vf0łȂ̂xɊi[
    if(sum(x) <= 0) {      #x̘a0ȉ̏ꍇ
        score <- log(x_length, base=2)
        return(rep(10000, x_length))
    } else {               #ȊȌꍇ
        p <- x/sum(x)      #xNgp̘a1ɂȂ悤ɃxNgx̘ałꂼ̗vf
        score <- -sum(p * log(p, base=2)) - log(p, base=2)
        x_org[x_org == 0] <- 10000
        tmp <- replace(x_org, ((x_org != 10000) & (!is.na(x_org))), score)
        return(tmp)      #Schug's Q-statistic̒lԂiANAȊO̗vfɂĂ̂݁j
    }
} 

#############################
###  Kadota's ROKU        ###
#############################
kadota_2006_bmc_bioinformatics <- function(x){ 
    x <- x[(!is.na(x))]                    #NAȊO̗vfxNgxɊi[
    x_length <- length(x)                  #xNgx̗vfx_lengthɊi[
    x <- x[(x != 0)]                       #vf0łȂ̂xɊi[
    if(sum(x) <= 0) {                      #x̘a0ȉ̏ꍇ
        return(log(x_length, base=2))      #ől^ 
    } else if(sd(x) == 0){                 #x̗vfS0̏ꍇ(2007/10/1ǉ)
        return(log(x_length, base=2))      #ől^
    } else {                               #ȊȌꍇ
        x_prime <- abs(x-tukey.biweight(x))#Tukey's biweightňāA̐ΒlƂ
        p <- x_prime/sum(x_prime)          #xNgp̘a1ɂȂ悤ɃxNgx̘ałꂼ̗vf
        e <- sum(p * log(p, base=2)) 
        return(-e)
    }
} 

##########################################################################
###  uKadota et al., Physiol. Genomics, 12(3), 251-259, 2003v̕@  ###
###  Ol␔sTv(n+s)25%Ɍ肵o[W           ###
###  uKadota et al., BMC Bioinformatics, 7, 294, 2006vō̗p         ###
##########################################################################
kadota_2003_physiol_genomics_0.25 <- function(x){ 
  # x̗vfSNANaN̏ꍇɃG[oĂ̂ŁÂ悤ȏꍇɂ͊OlȂAƂ(2007/10/4ǉ)
  if(length(x) == sum(is.na(x))){
    x <- c(rep(0, length(x)))
  } else if(length(x) == sum(is.nan(x))){
    x <- c(rep(0, length(x)))
  } else{
  }
  x_org <- x             #20090731ǉ
  x <- x[(!is.na(x))]    #NAȊO̗vfxNgxɊi[(20090731ǉ)
  x <- x[(!is.nan(x))]   #NaNȊO̗vfxNgxɊi[(20090731ǉ)

  n_plus_s <- length(x)  #xNgx̗vfx_lengthɊi[
  x.sort <- sort(x)
  x.order <- order(x)
  maice_Ut <- 0
  maice_i <- 0
  maice_j <- 0
  flag <- c(rep(0, length=n_plus_s))

  if(sd(x) != 0){ #x̗vfS0̏ꍇ(2007/10/1ǉ)
    for(i in 1:(n_plus_s*0.25+1)){  #2007/9/27n_plus_s -> (n_plus_s*0.25+1)ƕύX܂
      for(j in 1:(n_plus_s-i)){
        if((i+j-2)<=n_plus_s*0.25){
          n <- (n_plus_s+1-j) - i + 1
          s <- n_plus_s - n
          set_sd <- sd(x.sort[i:(n_plus_s+1-j)])*sqrt((n-1)/n)
          Ut <- n*log(set_sd) + sqrt(2)*s*(0.5*log(2*pi) + (0.5+n)*log(n)-n)/n
          if(maice_Ut > Ut){
            maice_Ut <- Ut
            maice_i <- i
            maice_j <- j
          }
          #tmp <- paste(i, n_plus_s+1-j, n, s, set_sd, Ut, sep=",")
          #print (tmp)
        }
      }
    }
    if(maice_i > 1){ flag[x.order[1:(maice_i-1)]] <- -1 }
    if(maice_j > 1){ flag[x.order[(n_plus_s+1-maice_j+1):n_plus_s]] <- 1 }

    tmp <- replace(x_org, ((!is.nan(x_org)) & (!is.na(x_org))), flag)  #20090731ǉ
    return(tmp)
  } else{
    tmp <- replace(x_org, ((!is.nan(x_org)) & (!is.na(x_org))), flag)  #20090731ǉ
    return(tmp)
  }
}

##########################################################################
###  uKadota et al., Physiol. Genomics, 12(3), 251-259, 2003v̕@  ###
###  Ol␔sTv(n+s)50%Ɍ肵o[W           ###
###  IWiy[p[ō̗p                                        ###
##########################################################################
kadota_2003_physiol_genomics_0.50 <- function(x){ 
  # x̗vfSNANaN̏ꍇɃG[oĂ̂ŁÂ悤ȏꍇɂ͊OlȂAƂ(2007/10/4ǉ)
  if(length(x) == sum(is.na(x))){
    x <- c(rep(0, length(x)))
  } else if(length(x) == sum(is.nan(x))){
    x <- c(rep(0, length(x)))
  } else{
  }
  x_org <- x             #20090731ǉ
  x <- x[(!is.na(x))]    #NAȊO̗vfxNgxɊi[(20090731ǉ)
  x <- x[(!is.nan(x))]   #NaNȊO̗vfxNgxɊi[(20090731ǉ)

  
  n_plus_s <- length(x)  #xNgx̗vfx_lengthɊi[
  x.sort <- sort(x)
  x.order <- order(x)
  maice_Ut <- 0
  maice_i <- 0
  maice_j <- 0
  flag <- c(rep(0, length=n_plus_s))

  if(sd(x) != 0){ #x̗vfS0̏ꍇ(2007/10/1ǉ)
    for(i in 1:(n_plus_s*0.50+1)){  #2007/9/27n_plus_s -> (n_plus_s*0.50+1)ƕύX܂
      for(j in 1:(n_plus_s-i)){
        if((i+j-2)<=n_plus_s*0.50){
          n <- (n_plus_s+1-j) - i + 1
          s <- n_plus_s - n
          set_sd <- sd(x.sort[i:(n_plus_s+1-j)])*sqrt((n-1)/n)
          Ut <- n*log(set_sd) + sqrt(2)*s*(0.5*log(2*pi) + (0.5+n)*log(n)-n)/n
          if(maice_Ut > Ut){
            maice_Ut <- Ut
            maice_i <- i
            maice_j <- j
          }
          #tmp <- paste(i, n_plus_s+1-j, n, s, set_sd, Ut, sep=",")
          #print (tmp)
        }
      }
    }
    if(maice_i > 1){ flag[x.order[1:(maice_i-1)]] <- -1 }
    if(maice_j > 1){ flag[x.order[(n_plus_s+1-maice_j+1):n_plus_s]] <- 1 }

    tmp <- replace(x_org, ((!is.nan(x_org)) & (!is.na(x_org))), flag)  #20090731ǉ
    return(tmp)
  } else{
    tmp <- replace(x_org, ((!is.nan(x_org)) & (!is.na(x_org))), flag)  #20090731ǉ
    return(tmp)
  }
}

